@@ -1,4 +1,4 @@
-use crate::fs::{Metadata, Permissions};
+use crate::fs::{Metadata, OpenOptions, Permissions};
 #[cfg(unix)]
 use async_std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 #[cfg(target_os = "wasi")]
@@ -8,13 +8,17 @@ use async_std::{
     io::{self, IoSlice, IoSliceMut, Read, Seek, SeekFrom, Write},
     task::{Context, Poll},
 };
-use cap_primitives::{ambient_authority, fs::is_file_read_write, AmbientAuthority};
+use cap_primitives::{
+    ambient_authority,
+    fs::{is_file_read_write, open_ambient},
+    AmbientAuthority,
+};
 use io_lifetimes::AsFilelike;
 #[cfg(not(windows))]
 use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
 #[cfg(windows)]
 use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
-use std::{fmt, pin::Pin};
+use std::{fmt, path::Path, pin::Pin};
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
 use {
@@ -38,12 +42,13 @@ pub struct File {
 }
 
 impl File {
-    /// Constructs a new instance of `Self` from the given `async_std::fs::File`.
+    /// Constructs a new instance of `Self` from the given
+    /// `async_std::fs::File`.
     ///
     /// # Ambient Authority
     ///
-    /// `async_std::fs::File` is not sandboxed and may access any path that the host
-    /// process has access to.
+    /// `async_std::fs::File` is not sandboxed and may access any path that the
+    /// host process has access to.
     #[inline]
     pub fn from_std(std: fs::File, _: AmbientAuthority) -> Self {
         Self { std }
@@ -65,8 +70,8 @@ impl File {
         self.std.sync_all().await
     }
 
-    /// This function is similar to `sync_all`, except that it may not synchronize
-    /// file metadata to a filesystem.
+    /// This function is similar to `sync_all`, except that it may not
+    /// synchronize file metadata to a filesystem.
     ///
     /// This corresponds to [`async_std::fs::File::sync_data`].
     #[inline]
@@ -74,8 +79,8 @@ impl File {
         self.std.sync_data().await
     }
 
-    /// Truncates or extends the underlying file, updating the size of this file
-    /// to become size.
+    /// Truncates or extends the underlying file, updating the size of this
+    /// file to become size.
     ///
     /// This corresponds to [`async_std::fs::File::set_len`].
     #[inline]
@@ -104,6 +109,44 @@ impl File {
             .set_permissions(permissions_into_std(&sync, perm)?)
             .await
     }
+
+    /// Constructs a new instance of `Self` in read-only mode by opening the
+    /// given path as a file using the host process' ambient authority.
+    ///
+    /// # Ambient Authority
+    ///
+    /// This function is not sandboxed and may access any path that the host
+    /// process has access to.
+    #[inline]
+    pub fn open_ambient<P: AsRef<Path>>(
+        path: P,
+        ambient_authority: AmbientAuthority,
+    ) -> io::Result<Self> {
+        open_ambient(
+            path.as_ref(),
+            &OpenOptions::new().read(true),
+            ambient_authority,
+        )
+        .map(|f| Self::from_std(f.into(), ambient_authority))
+    }
+
+    /// Constructs a new instance of `Self` with the options specified by
+    /// `options` by opening the given path as a file using the host process'
+    /// ambient authority.
+    ///
+    /// # Ambient Authority
+    ///
+    /// This function is not sandboxed and may access any path that the host
+    /// process has access to.
+    #[inline]
+    pub fn open_ambient_with<P: AsRef<Path>>(
+        path: P,
+        options: &OpenOptions,
+        ambient_authority: AmbientAuthority,
+    ) -> io::Result<Self> {
+        open_ambient(path.as_ref(), options, ambient_authority)
+            .map(|f| Self::from_std(f.into(), ambient_authority))
+    }
 }
 
 #[cfg(not(target_os = "wasi"))]
